import sys
import getopt
import os
import os.path
import signal
import gettext
import gtk
import gnome
import gobject
import gtk.glade
import shutil
import commands

from pxeos import *
from netboot_util import *
from netboot_gtk import *
import diskless

#Columns for "treeview"
COLUMN_OS=0
COLUMN_DESCRIPTION=1
COLUMN_DATA=2

NFS=0
HTTP=1
FTP=2
PROTOCOL={ NFS : "NFS", FTP : "FTP", HTTP : "HTTP" }

def translateProtocol(p):
    if p =="NFS":
        return NFS
    if p =="FTP":
        return FTP
    if p =="HTTP":
        return HTTP
        
class pxelistdialog:

    def __init__(self,xml):
        self.xml=xml
        self.configureOSDialog=self.xml.get_widget("configureOSDialog")
        self.xml.get_widget("addConfigureOSButton").connect("clicked",self.on_addOSButton_clicked)
        self.xml.get_widget("deleteConfigureOSButton").connect("clicked",self.on_deleteOSButton_clicked)
        self.xml.get_widget("propertiesConfigureOSButton").connect("clicked",self.on_propertiesOSButton_clicked)
        self.pxeosDialog=pxeosdialog(self.xml)
        self.disklessDialog = diskless.disklessDialog(self.xml)
        self.setupList()
        self.OS_name_changes = {}

    def on_edit(self,OS=None,iter=None):
        if OS == None:
            OS=pxeos()
        orig_name = OS.getName()
        if self.useDiskless:
            rc,OS=self.disklessDialog.run(OS)
        else:
            rc,OS=self.pxeosDialog.run(OS)
        if rc==gtk.RESPONSE_OK:
            model=self.tree_view.get_model ()
            if iter==None:
                iter = model.append ()
            model.set (iter,
                       COLUMN_OS,OS.getName(),
                       COLUMN_DESCRIPTION,OS.getDescription(),
                       COLUMN_DATA, OS)
            self.tree_view.get_selection().select_iter (iter)
            if OS.getName() != orig_name:
                self.OS_name_changes[orig_name] = OS.getName()
            return OS
        else:
            return None
        
            
    def on_propertiesOSButton_clicked(self,*args):
        """Callback for the propertiesButton."""
        result = self.tree_view.get_selection().get_selected ()
        if result == None:
            return 
        else:
            (model, iter) = result
            try:
                os = model.get_value (iter, COLUMN_DATA)
            except TypeError:
                return
            self.on_edit(os,iter)
        
    def on_deleteOSButton_clicked(self,*args):
        """Callback for the deleteButton."""
        result = self.tree_view.get_selection().get_selected ()
        if result == None:
            return 
        else:
            (model, iter) = result
            try:
                os = model.get_value (iter, COLUMN_DATA)
            except TypeError:
                return
            message=_("Are you sure you want to delete the '%s' Operating System config?") % os.getName()
            dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO,
                                    gtk.BUTTONS_YES_NO,
                                    message)
            dlg.show_all()
            rc = dlg.run()
            dlg.destroy()
            if rc==gtk.RESPONSE_YES:
                model.remove(iter)
    
    def runDiskless(self):
        self.useDiskless=TRUE
        return self.on_edit()

    def runNetboot(self):
        self.useDiskless=FALSE
        return self.on_edit()

    def on_addOSButton_clicked(self,*args):
        self.on_edit()

    def setupList(self):
        """Populates osTreeview with the os files"""
        self.tree_view = self.xml.get_widget("configureOSTreeview")
        self.model = gtk.ListStore (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_PYOBJECT)
        self.model.set_sort_column_id(1, gtk.SORT_ASCENDING)        
        self.tree_view.set_model (self.model)
        column = gtk.TreeViewColumn (_("OS Name"), gtk.CellRendererText(), text=COLUMN_OS)
        column.set_sort_column_id(1)
        self.tree_view.append_column (column)
        column = gtk.TreeViewColumn (_("Description"), gtk.CellRendererText(), text=COLUMN_DESCRIPTION)
        column.set_sort_column_id(2)
        self.tree_view.append_column (column)
        self.tree_view.connect ("row_activated",self.on_tree_view_row_activated,
                               self.on_propertiesOSButton_clicked)

    def populateList(self):
        self.model.clear()
        osList=[]
        for OS in pxeOSList().getOSList():
            if (OS.isDiskless() and self.useDiskless) or (not OS.isDiskless() and not self.useDiskless):
                iter = self.model.append ()
                self.model.set (iter,
                                COLUMN_OS,OS.getName(),
                                COLUMN_DESCRIPTION,OS.getDescription(),
                                COLUMN_DATA, OS)
            else:
                osList.append(OS)

        selection=self.tree_view.get_selection()
        selection.select_path ((0,))
        return osList

    def on_tree_view_row_activated (self,tree_view, path, column, func):
        apply (func)

    def run(self,useDiskless=FALSE):
        self.useDiskless=useDiskless
        if self.useDiskless:
            self.configureOSDialog.set_title(_("Configure Diskless Environment"))
        else:
            self.configureOSDialog.set_title(_("Configure Network Installations"))
            
        osList=self.populateList()
        self.configureOSDialog.run()
        self.configureOSDialog.hide()

        def foreach_func (model, path, iter, osList):
            OS = model.get_value (iter, COLUMN_DATA)
            osList.append(OS)
            return FALSE
        self.model.foreach (foreach_func, osList)
        pxeOSList().saveOSList(osList)

class pxeosdialog:
    def __init__(self,xml):
        self.xml = xml
        self.osName = self.xml.get_widget("osNameEntry")
        self.osDescription = self.xml.get_widget("osDescriptionEntry")
        self.kickstart = self.xml.get_widget("osKickstartEntry")
        self.protocol=self.xml.get_widget("osProtocolOptionmenu")
        self.location = self.xml.get_widget("locationEntry")
        self.server = self.xml.get_widget("serverEntry")
        self.user=self.xml.get_widget("userFTPEntry")
        self.password=self.xml.get_widget("passwordFTPEntry")
        self.anonymousCheckbutton=self.xml.get_widget("anonymousCheckbutton")
        self.anonymousCheckbutton.connect("toggled",self.on_anonymousCheckbutton_toggled)
        self.protocol.connect("changed",self.on_protocol_changed)
        self.server.set_text(hostname())

    def on_edit(self,OS=None,iter=None):
        model=self.tree_view.get_model ()
        if OS == None:
            OS=pxeos()
        self.osName.set_text(OS.getName())
        self.osDescription.set_text(OS.getDescription())
        self.kickstart.set_text(OS.getKickstart())
        self.protocol.set_history(translateProtocol(OS.getProtocol()))
        self.location.set_text(OS.getLocation())
        self.server.set_text(OS.getServer())
        self.anonymousCheckbutton.set_active(OS.isAnonymous())
        self.on_protocol_changed()
        self.user.set_text(OS.getUser())
        self.password.set_text(OS.getPassword())
        
        dlg = self.xml.get_widget("osDialog")
        dlg.show_all()
        while 1:
            rc = dlg.run()
            if rc==gtk.RESPONSE_OK:
                if self.osName.get_text()=="":
                    errorMessage(_("You must enter a Operating System Name"))
                    continue

                protocol = self.protocol.get_history()
                location = self.location.get_text()
                server = self.server.get_text()

                OS=pxeos()
                OS.setName(self.osName.get_text())
                OS.setDescription(self.osDescription.get_text())
                OS.setKickstart(self.kickstart.get_text())
                OS.setProtocol(PROTOCOL[protocol])
                OS.setDiskless(FALSE)
                OS.setLocation(location)
                OS.setAnonymous(self.anonymousCheckbutton.get_active())
                OS.setUser(self.user.get_text())
                OS.setPassword(self.password.get_text())
                    
                try:
                    OS.updatePXE()
                except IOError,e:
                    errorMessage(_("You must enter directory of the Operating System Software\nThis directory must include the images/pxeboot directories\n%s" % e))
                    continue
                    
                if iter==None:
                    iter = model.append ()
                model.set (iter,
                           COLUMN_OS,OS.getName(),
                           COLUMN_DESCRIPTION,OS.getDescription(),
                           COLUMN_DATA, OS)
                
            dlg.hide()
            break

    def on_anonymousCheckbutton_toggled(self,button):
        self.user.set_sensitive(not button.get_active())         
        self.password.set_sensitive(not button.get_active())         

    def on_protocol_changed(self,*args):
        if self.protocol.get_history()==FTP:
            self.xml.get_widget("anonymousCheckbutton").set_sensitive(TRUE)
            self.on_anonymousCheckbutton_toggled(self.anonymousCheckbutton)
        else:
            self.anonymousCheckbutton.set_sensitive(FALSE)
            self.user.set_sensitive(FALSE)         
            self.password.set_sensitive(FALSE)
        
    def run(self,OS):
        self.osName.set_text(OS.getName())
        self.osDescription.set_text(OS.getDescription())
        self.kickstart.set_text(OS.getKickstart())
        self.protocol.set_history(translateProtocol(OS.getProtocol()))
        self.location.set_text(OS.getLocation())
        self.server.set_text(OS.getServer())
        self.anonymousCheckbutton.set_active(OS.isAnonymous())
        self.on_protocol_changed()
        self.user.set_text(OS.getUser())
        self.password.set_text(OS.getPassword())

        dlg = self.xml.get_widget("osDialog")
        dlg.show_all()
        while 1:
            rc = dlg.run()
            if rc==gtk.RESPONSE_OK:
                if self.osName.get_text()=="":
                    errorMessage(_("You must enter a Operating System Name"))
                    continue

                if self.osName.get_text() in [ OS.getName() for OS in pxeOSList().getOSList() ]:
                    errorMessage(_("Name '%s' already exists. Please choose other name.")
                        % self.osName.get_text())
                    continue

                protocol = self.protocol.get_history()
                location = self.location.get_text()
                server = self.server.get_text()

                OS.setName(self.osName.get_text())
                OS.setDescription(self.osDescription.get_text())
                OS.setKickstart(self.kickstart.get_text())
                OS.setProtocol(PROTOCOL[protocol])
                OS.setDiskless(FALSE)
                OS.setServer(server)
                OS.setLocation(location)
                OS.setAnonymous(self.anonymousCheckbutton.get_active())
                OS.setUser(self.user.get_text())
                OS.setPassword(self.password.get_text())
                try:
                    OS.updatePXE()
                except IOError,e:
                    errorMessage(_("You must enter directory of the Operating System Software\nThis directory must include the images/pxeboot directories\n%s" % e))
                    continue
                    
            break
        dlg.hide()
        return rc,OS




